#!/usr/bin/env node


'use strict';

var Promise = require("bluebird");
var chokidar = require("chokidar");
var fs = Promise.promisifyAll(require("fs"));
var path = require("path");
var stacktraceParser = require("stacktrace-parser");

var loadConfiguration = require("./load-configuration");
var printError = require("./print-error");
var createEmailSender = require("./email/create-sender");
var generateEmailPreview = require("./email/generate-preview");
var readError = require("./error/read");
var parseError = require("./error/parse");

var config = loadConfiguration(process.argv[2]);

var transport = createEmailSender(config);

function reportError(errorFilePath) {
    Promise.try(function () {
        return readError(errorFilePath);
    }).then(function (errorData) {
        var error = parseError(errorData.error);

        // FIXME: Strip everything after the first line of the message?
        var subjectLine = config.subjectFormat.replace(/\$type/g, error.name).replace(/\$message/g, error.message.replace(/\n/g, " "));

        return Promise.try(function () {
            return transport.sendMail({
                subject: subjectLine,
                text: generateEmailPreview({
                    environmentName: errorData.environmentName,
                    hostname: errorData.hostname,
                    error: error
                }, config.stackFilter),
                attachments: [{
                    filename: path.basename(errorFilePath),
                    content: JSON.stringify(Object.assign(errorData, {
                        parsedStack: error.stack,
                        simplifiedStack: error.stackWithout("*", true)
                    }), null, "\t")
                }]
            });
        }).then(function () {
            console.log("Reported error in " + path.basename(errorFilePath) + ": " + error.name + " - " + error.message);
        });
    }).catch(function (err) {
        printError(["ERROR: Unable to parse or report error for " + errorFilePath + ".", err]);
    });
}

chokidar.watch(config.errorPath, { ignoreInitial: true }).on("add", function (errorFilePath) {
    reportError(errorFilePath);
}).on("ready", function () {
    console.log("Listening for new errors...");
});